(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', './adapters/FormData'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('./adapters/FormData'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.FormData);
    global.FormObject = mod.exports;
  }
})(this, function (exports, _FormData) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _FormData2 = _interopRequireDefault(_FormData);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _defineProperty(obj, key, value) {
    if (key in obj) {
      Object.defineProperty(obj, key, {
        value: value,
        enumerable: true,
        configurable: true,
        writable: true
      });
    } else {
      obj[key] = value;
    }

    return obj;
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var FormObject = function () {
    function FormObject() {
      var initialData = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      _classCallCheck(this, FormObject);

      this.id = initialData.id;

      var _constructor = this.constructor;
      var submodelProperties = _constructor.submodelProperties;
      var properties = _constructor.properties;

      this.attributes = { errors: {} };

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = properties[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var _property = _step.value;

          this.attributes[_property] = null;
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      for (var submodel in submodelProperties) {
        this.attributes[submodel] = {};
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = submodelProperties[submodel][Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var property = _step2.value;

            this.attributes[submodel][property] = null;
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = Object.keys(initialData)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var _field = _step3.value;

          if (properties.includes(_field)) {
            this.attributes[_field] = initialData[_field];
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      for (var _submodel in submodelProperties) {
        if (initialData.hasOwnProperty(_submodel)) {
          for (var field in initialData[_submodel]) {
            if (submodelProperties[_submodel].includes(field)) {
              this.attributes[_submodel][field] = initialData[_submodel][field];
            }
          }
        }
      }
    }

    // --- Overwritable --- //

    _createClass(FormObject, [{
      key: 'validation',
      value: function validation() {}
    }, {
      key: 'validate',
      value: function validate(attribute) {

        if (!this.attributes[attribute]) {
          return _defineProperty({}, attribute, ['must be filled']);
        }
        return {};
      }
    }, {
      key: 'adapter',
      value: function adapter(form) {
        return this._adapter || new this.constructor.ajaxAdapter(form, this);
      }
    }, {
      key: 'requestHash',
      value: function requestHash(form) {
        return this.adapter(form).requestHash;
      }
    }, {
      key: 'handleAjaxResponse',
      value: function handleAjaxResponse(json) {
        return this.adapter().handleAjaxResponse(json);
      }
    }, {
      key: 'attributes',
      get: function get() {
        return this._attributes;
      },
      set: function set(attributes) {
        this._attributes = attributes;
      }
    }], [{
      key: 'model',
      get: function get() {
        throw 'FormObject ' + this.name + ' needs to define a model name.';
      }
    }, {
      key: 'properties',
      get: function get() {
        throw 'FormObject ' + this.name + ' needs to define properties.';
      }
    }, {
      key: 'submodels',
      get: function get() {
        return [];
      }
    }, {
      key: 'submodelProperties',
      get: function get() {
        return {};
      }
    }, {
      key: 'ajaxAdapter',
      get: function get() {
        return _FormData2.default;
      }
    }]);

    return FormObject;
  }();

  exports.default = FormObject;
});