(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', './setFormSaved'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('./setFormSaved'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.setFormSaved);
    global.submitAjaxForm = mod.exports;
  }
})(this, function (exports, _setFormSaved) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.default = submitAjaxForm;

  var _setFormSaved2 = _interopRequireDefault(_setFormSaved);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _toConsumableArray(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) {
        arr2[i] = arr[i];
      }

      return arr2;
    } else {
      return Array.from(arr);
    }
  }

  var submitAjaxFormRequest = function submitAjaxFormRequest(formId) {
    return {
      type: '_RFORM_SUBMIT_AJAX_FORM_REQUEST',
      formId: formId
    };
  };

  var submitAjaxFormReturn = function submitAjaxFormReturn(formId) {
    return {
      type: '_RFORM_SUBMIT_AJAX_FORM_RETURN',
      formId: formId
    };
  };

  var handleAjaxResponse = function handleAjaxResponse(_changes, formErrorHash, _meta) {
    return {
      type: '_RFORM_HANDLE_AJAX_RESPONSE',
      formErrorHash: formErrorHash
    };
  };

  function submitAjaxForm(formId, url, form, formObject, afterResponse) {
    var afterError = arguments.length <= 5 || arguments[5] === undefined ? function () {} : arguments[5];

    return function (dispatch) {
      dispatch(submitAjaxFormRequest(formId));

      //const fetch = require('isomorphic-fetch') // regular import breaks in SSR
      return fetch(url, formObject.requestHash(form)).then(function (response) {
        var status = response.status;
        var statusText = response.statusText;

        dispatch(submitAjaxFormReturn(formId));
        if (status >= 400) {
          console.error('Submit Ajax Form Error ' + status + ': ' + statusText);
          afterError(formId, response.clone());
        }
        return response.json();
      }).then(function (json) {
        var evaluatedResponseContents = formObject.handleAjaxResponse(json);
        // if (handleResponse) {
        //   // TODO: Do we still need both handleResponse AND afterResponse?
        //   handleResponse(formId, ...evaluatedResponseContents, json)
        // }

        dispatch(handleAjaxResponse.apply(undefined, _toConsumableArray(evaluatedResponseContents)));

        var responseErrors = evaluatedResponseContents[1];
        if (!responseErrors || !Object.keys(responseErrors).length) dispatch((0, _setFormSaved2.default)(formId));

        if (afterResponse) afterResponse.apply(undefined, [formId].concat(_toConsumableArray(evaluatedResponseContents), [json]));
      });
    };
  }
});