(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react-redux', 'lodash/merge', 'lodash/forIn', 'react', '../actions/setupAction', '../actions/updateAction', '../actions/submitAjaxForm', '../utils/defaultFormObject', '../utils/validate', '../actions/updateError', '../components/Form'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react-redux'), require('lodash/merge'), require('lodash/forIn'), require('react'), require('../actions/setupAction'), require('../actions/updateAction'), require('../actions/submitAjaxForm'), require('../utils/defaultFormObject'), require('../utils/validate'), require('../actions/updateError'), require('../components/Form'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.reactRedux, global.merge, global.forIn, global.react, global.setupAction, global.updateAction, global.submitAjaxForm, global.defaultFormObject, global.validate, global.updateError, global.Form);
    global.Form = mod.exports;
  }
})(this, function (exports, _reactRedux, _merge, _forIn, _react, _setupAction, _updateAction, _submitAjaxForm, _defaultFormObject, _validate, _updateError, _Form) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _merge2 = _interopRequireDefault(_merge);

  var _forIn2 = _interopRequireDefault(_forIn);

  var _setupAction2 = _interopRequireDefault(_setupAction);

  var _updateAction2 = _interopRequireDefault(_updateAction);

  var _submitAjaxForm2 = _interopRequireDefault(_submitAjaxForm);

  var _defaultFormObject2 = _interopRequireDefault(_defaultFormObject);

  var _updateError2 = _interopRequireDefault(_updateError);

  var _Form2 = _interopRequireDefault(_Form);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  var mapStateToProps = function mapStateToProps(state, ownProps) {
    var formObjectClass = ownProps.formObjectClass || (0, _defaultFormObject2.default)(ownProps.model, ownProps.children);
    var formId = ownProps.id || formObjectClass.name;
    var editedStateObject = state.rform[formId];
    var rformState = state.rform;

    var enctype = ownProps.multipart ? 'multipart/form-data' : 'application/x-www-form-urlencoded';

    var formMethod = ownProps.method || 'POST';
    var hiddenMethod = formMethod;
    if (['GET', 'POST'].indexOf(formMethod) < 0) {
      // is non-standard method
      formMethod = 'POST';
    }

    var authToken = ownProps.authToken || state.authToken || state.settings && state.settings.authToken;
    var model = ownProps.model || formObjectClass.model;

    return {
      existingAttrs: assembleAttrsFromServer(ownProps.seedData, formObjectClass),
      editedStateObject: editedStateObject,
      rformState: rformState,
      formId: formId,
      commit: editedStateObject && editedStateObject.commit,
      enctype: enctype,
      formMethod: formMethod,
      hiddenMethod: hiddenMethod,
      authToken: authToken,
      combinedClassName: ['rform-form', ownProps.className].join(' '),
      model: model,
      formObjectClass: formObjectClass
    };
  };

  // the values we want from the server-provided serialized object are nested
  // under `fields`. The same is true for included submodels.
  function assembleAttrsFromServer(seedData, formObjectClass) {
    // Early return when there was no serialized object provided by the server
    if (!seedData) {
      return {};
    }

    // Otherwise assemble main and submodels' fields
    var attrs = (0, _merge2.default)({}, seedData.fields, seedData.errors);
    // TODO: reenable this, but consider there might be ids/ id arrays in there
    // for (let submodel of formObjectClass.submodels) {
    //   if (seedData.fields[submodel]) {
    //     let submodelData = seedData.fields[submodel]
    //     attrs[submodel] = merge({}, submodelData.fields, submodelData.errors)
    //   }
    // }
    return attrs;
  }

  var mapDispatchToProps = function mapDispatchToProps(dispatch) {
    return {
      onBlur: function onBlur() {
        // TODO: validationAction not yet defined
        // return dispatch(validationAction(field))
      },


      dispatch: dispatch
    };
  };

  var mergeProps = function mergeProps(stateProps, dispatchProps, ownProps) {
    return _extends({}, ownProps, stateProps, dispatchProps, {
      onSubmit: function onSubmit(event) {
        if (!ownProps.ajax) return true;
        if (event.preventDefault) event.preventDefault();

        var dispatch = dispatchProps.dispatch;
        var formObjectClass = stateProps.formObjectClass;
        var formId = stateProps.formId;
        var rformState = stateProps.rformState;
        var afterResponse = ownProps.afterResponse;
        var afterRequireValid = ownProps.afterRequireValid;
        var beforeSubmit = ownProps.beforeSubmit;
        var afterError = ownProps.afterError;


        var formObject = new formObjectClass(rformState, formId);
        if (ownProps.requireValid) {
          var result = (0, _validate.validateForm)(formObject);
          if (afterRequireValid) afterRequireValid(result);
          if (!result.valid) {
            (0, _forIn2.default)(result.errors, function (errors, erroringFormId) {
              return dispatch((0, _updateError2.default)(erroringFormId, errors));
            });
            return false;
          }
        }

        if (beforeSubmit) beforeSubmit();

        dispatch((0, _submitAjaxForm2.default)(formId, ownProps.action, event.target, formObject, afterResponse, afterError));
        return false;
      },
      ensureStateObjectExistence: function ensureStateObjectExistence() {
        if (stateProps.editedStateObject) return;
        //  if there is no state object yet, create one
        dispatchProps.dispatch((0, _setupAction2.default)(stateProps.formId, stateProps.existingAttrs));

        // call initial data setting callback
        ownProps.formStateDidMount && ownProps.formStateDidMount(stateProps.formId, stateProps.existingAttrs);
      }
    });
  };

  exports.default = (0, _reactRedux.connect)(mapStateToProps, mapDispatchToProps, mergeProps)(_Form2.default);
});