(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react-redux', 'lodash/isNil', '../utils/modelParams', '../utils/translations', '../utils/stateNavigation', '../utils/getErrors', '../actions/updateError', '../actions/updateAction', '../components/Input'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react-redux'), require('lodash/isNil'), require('../utils/modelParams'), require('../utils/translations'), require('../utils/stateNavigation'), require('../utils/getErrors'), require('../actions/updateError'), require('../actions/updateAction'), require('../components/Input'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.reactRedux, global.isNil, global.modelParams, global.translations, global.stateNavigation, global.getErrors, global.updateError, global.updateAction, global.Input);
    global.Input = mod.exports;
  }
})(this, function (exports, _reactRedux, _isNil, _modelParams, _translations, _stateNavigation, _getErrors, _updateError, _updateAction, _Input) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _isNil2 = _interopRequireDefault(_isNil);

  var _updateError2 = _interopRequireDefault(_updateError);

  var _updateAction2 = _interopRequireDefault(_updateAction);

  var _Input2 = _interopRequireDefault(_Input);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  function _toConsumableArray(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) {
        arr2[i] = arr[i];
      }

      return arr2;
    } else {
      return Array.from(arr);
    }
  }

  var mapStateToProps = function mapStateToProps(state, ownProps) {
    var formId = ownProps.formId;
    var model = ownProps.model;
    var attribute = ownProps.attribute;
    var className = ownProps.className;
    var disabled = ownProps.disabled;


    var submodelPath = ownProps.submodelPath || [];

    var rformState = state.rform;
    var attrs = rformState[formId];
    var path = (0, _stateNavigation.navigateThroughSubmodels)(attrs, submodelPath);
    var value = '';
    if (path && !(0, _isNil2.default)(path[attribute])) value = path[attribute];

    var savedValue = attrs && attrs._savedAttributes && attrs._savedAttributes[attribute];

    var name = (0, _modelParams.getName)(model, attribute, submodelPath);
    var inputId = (0, _modelParams.getId)(formId, model, attribute, submodelPath);

    var placeholder = ownProps.placeholder || _translations.optionalTranslation.apply(undefined, ['rform', model].concat(_toConsumableArray(submodelPath), [attribute, 'placeholder']));

    var combinedClassName = ['input-' + attribute, className].join(' ');

    return {
      value: value,
      savedValue: savedValue,
      name: name,
      inputId: inputId,
      placeholder: placeholder,
      combinedClassName: combinedClassName,
      disabled: disabled === undefined ? false : disabled,
      formState: attrs,
      rformState: rformState
    };
  };

  var mapDispatchToProps = function mapDispatchToProps(dispatch) {
    return {
      dispatch: dispatch
    };
  };

  var mergeProps = function mergeProps(stateProps, dispatchProps, ownProps) {
    var validate = function validate() {
      var attribute = ownProps.attribute;
      var formObjectClass = ownProps.formObjectClass;
      var submodelPath = ownProps.submodelPath;
      var formId = ownProps.formId;
      var formState = stateProps.formState;
      var rformState = stateProps.rformState;


      var formObject = new formObjectClass(rformState, formId);
      formObject.validate(attribute);
      var errKey = (0, _getErrors.errorKey)(attribute, submodelPath);
      var errors = formObject.attributes[_getErrors.ERRORCONTAINER];

      if (!errors && (!formState.errors || !formState.errors[errKey])) return;
      dispatchProps.dispatch((0, _updateError2.default)(formId, errors, errKey));
    };

    return _extends({}, ownProps, stateProps, dispatchProps, {
      onChange: function onChange(event) {
        var formId = ownProps.formId;
        var attribute = ownProps.attribute;
        var submodelPath = ownProps.submodelPath;
        var preventEnterSubmit = ownProps.preventEnterSubmit;
        var savedValue = stateProps.savedValue;

        var newValue = event.target.value;

        dispatchProps.dispatch((0, _updateAction2.default)(formId, attribute, submodelPath, newValue));

        if (ownProps.submitOnChange) {
          ownProps.onSubmit({ target: ownProps.form });
        }

        if (ownProps.afterChange) ownProps.afterChange(event);
      },
      onBlur: function onBlur(_event) {
        validate();
      },
      onKeyPress: function onKeyPress(event) {
        if (ownProps.preventEnterSubmit && event.charCode == 13) {
          event.preventDefault();
          return false;
        }
        return true;
      },


      validate: validate
    });
  };

  exports.default = (0, _reactRedux.connect)(mapStateToProps, mapDispatchToProps, mergeProps)(_Input2.default);
});