(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'react-redux', '../utils/modelParams', '../utils/getErrors', '../utils/stateNavigation', '../utils/translations', '../components/InputSet'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('react-redux'), require('../utils/modelParams'), require('../utils/getErrors'), require('../utils/stateNavigation'), require('../utils/translations'), require('../components/InputSet'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.reactRedux, global.modelParams, global.getErrors, global.stateNavigation, global.translations, global.InputSet);
    global.InputSet = mod.exports;
  }
})(this, function (exports, _reactRedux, _modelParams, _getErrors, _stateNavigation, _translations, _InputSet) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _getErrors2 = _interopRequireDefault(_getErrors);

  var _InputSet2 = _interopRequireDefault(_InputSet);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _toConsumableArray(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) {
        arr2[i] = arr[i];
      }

      return arr2;
    } else {
      return Array.from(arr);
    }
  }

  var mapStateToProps = function mapStateToProps(state, ownProps) {
    var attribute = ownProps.attribute;
    var submodelPath = ownProps.submodelPath;
    var wrapperClassName = ownProps.wrapperClassName;
    var wrapperErrorClassName = ownProps.wrapperErrorClassName;
    var label = ownProps.label;
    var formId = ownProps.formId;
    var model = ownProps.model;


    var formState = state.rform[formId];
    var errors = (0, _getErrors2.default)(formState, attribute, submodelPath, ownProps.errors);

    var changeList = formState && (0, _stateNavigation.navigateThroughSubmodels)(formState._changes, submodelPath);
    var changeClass = changeList && changeList.includes(attribute) ? 'changed' : 'saved';

    var combinedWrapperClassName = ['inputset-' + attribute, wrapperClassName, changeClass].join(' ');
    var errorClass = wrapperErrorClassName || 'has-errors';
    if (errors && errors.length) combinedWrapperClassName += ' ' + errorClass;

    var labelText = label || _translations.optionalTranslation.apply(undefined, ['rform', model].concat(_toConsumableArray(submodelPath), [attribute, 'label'])) || '';

    return {
      combinedWrapperClassName: combinedWrapperClassName,
      labelText: labelText,
      errors: errors
    };
  };

  var mapDispatchToProps = function mapDispatchToProps(dispatch) {
    return {};
  };

  exports.default = (0, _reactRedux.connect)(mapStateToProps, mapDispatchToProps)(_InputSet2.default);
});