(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'lodash/isArray', './adapters/FormData', './Validation', '../utils/mixin', '../utils/getErrors'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('lodash/isArray'), require('./adapters/FormData'), require('./Validation'), require('../utils/mixin'), require('../utils/getErrors'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.isArray, global.FormData, global.Validation, global.mixin, global.getErrors);
    global.FormObject = mod.exports;
  }
})(this, function (exports, _isArray, _FormData, _Validation, _mixin, _getErrors) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _isArray2 = _interopRequireDefault(_isArray);

  var _FormData2 = _interopRequireDefault(_FormData);

  var _Validation2 = _interopRequireDefault(_Validation);

  var _mixin2 = _interopRequireDefault(_mixin);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _toConsumableArray(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) {
        arr2[i] = arr[i];
      }

      return arr2;
    } else {
      return Array.from(arr);
    }
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var FormObject = function () {
    function FormObject(rformData, formId) {
      _classCallCheck(this, FormObject);

      this.formId = formId;
      this.rformData = rformData;

      var initialData = rformData[formId] || {};
      this.initialData = initialData;
      this.id = initialData.id;

      var _constructor = this.constructor;
      var submodels = _constructor.submodels;
      var submodelConfig = _constructor.submodelConfig;
      var properties = _constructor.properties;

      this.attributes = {};
      this.attributes[_getErrors.ERRORCONTAINER] = {};

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = properties[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var property = _step.value;

          this.attributes[property] = null;
        }

        // initialize empty submodel properties
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = submodels[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var initialSubmodel = _step2.value;

          this.attributes[initialSubmodel] = [];
          // let initialConfig = this._submodelConfig(initialSubmodel)
          // if (initialConfig.relationship == 'oneToOne') {
          //   for (let property of initialConfig.properties) {
          //     this.attributes[initialSubmodel][property] = null
          //   }
          // }
        }

        // transfer own properties
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = Object.keys(initialData)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var field = _step3.value;

          if (properties.includes(field)) {
            this.attributes[field] = initialData[field];
          }
        }

        // transfer submodel properties
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }

      var _iteratorNormalCompletion4 = true;
      var _didIteratorError4 = false;
      var _iteratorError4 = undefined;

      try {
        for (var _iterator4 = submodels[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
          var submodel = _step4.value;

          var config = this.constructor._submodelConfig(submodel);

          var submodelDataSets = this._collectSubmodelDataSets(submodel, rformData, initialData, config);
          this.attributes[submodel] = submodelDataSets;

          // for (let fieldOrIndex of initialData[submodel]) { // each submodel prop
          //   if (fieldOrIndex.match(/^\d+$/)) { // is index, consists of number(s)
          //     this.attributes[submodel][fieldOrIndex] = {}
          //     for (let field in initialData[submodel][fieldOrIndex]) {
          //       if (config.properties.includes(field)) {
          //         this.attributes[submodel][fieldOrIndex][field] =
          //           initialData[submodel][fieldOrIndex][field]
          //       }
          //     }
          //   } else if (config.properties.includes(fieldOrIndex)) { // is non-number
          //     this.attributes[submodel][fieldOrIndex] =
          //       initialData[submodel][fieldOrIndex]
          //   }
          // }
        }
      } catch (err) {
        _didIteratorError4 = true;
        _iteratorError4 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion4 && _iterator4.return) {
            _iterator4.return();
          }
        } finally {
          if (_didIteratorError4) {
            throw _iteratorError4;
          }
        }
      }
    }

    // --- Overwritable --- //

    _createClass(FormObject, [{
      key: 'validation',
      value: function validation() {}
    }, {
      key: 'adapter',
      value: function adapter(form) {
        return this._adapter || new this.constructor.ajaxAdapter(form, this);
      }
    }, {
      key: 'requestHash',
      value: function requestHash(form) {
        return this.adapter(form).requestHash;
      }
    }, {
      key: 'handleAjaxResponse',
      value: function handleAjaxResponse(json) {
        return this.adapter().handleAjaxResponse(json);
      }
    }, {
      key: '_registeredSubFormsFor',
      value: function _registeredSubFormsFor(submodel, initialData) {
        return initialData._registeredSubmodelForms && initialData._registeredSubmodelForms[submodel] || [];
      }
    }, {
      key: '_collectSubmodelDataSets',
      value: function _collectSubmodelDataSets(submodel, rformData, initialData, config) {
        var dataSets = [];

        // form uses external form as submodel form
        var _iteratorNormalCompletion5 = true;
        var _didIteratorError5 = false;
        var _iteratorError5 = undefined;

        try {
          for (var _iterator5 = this._registeredSubFormsFor(submodel, initialData)[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
            var formId = _step5.value;

            dataSets.push(new config.object(rformData, formId).attributes);
          }
        } catch (err) {
          _didIteratorError5 = true;
          _iteratorError5 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion5 && _iterator5.return) {
              _iterator5.return();
            }
          } finally {
            if (_didIteratorError5) {
              throw _iteratorError5;
            }
          }
        }

        if (initialData.hasOwnProperty(submodel)) {
          // form has internal nested submodel(s)
          if ((0, _isArray2.default)(initialData[submodel])) {
            var _dataSets;

            (_dataSets = dataSets).push.apply(_dataSets, _toConsumableArray(initialData[submodel]));
          } else if (initialData[submodel]) {
            dataSets = [initialData[submodel]];
          }
        }
        return dataSets;
      }
    }, {
      key: 'attributes',
      get: function get() {
        return this._attributes;
      },
      set: function set(attributes) {
        this._attributes = attributes;
      }
    }, {
      key: 'submodelFormObjects',
      get: function get() {
        var objects = {};

        var _iteratorNormalCompletion6 = true;
        var _didIteratorError6 = false;
        var _iteratorError6 = undefined;

        try {
          for (var _iterator6 = this.constructor.submodels[Symbol.iterator](), _step6; !(_iteratorNormalCompletion6 = (_step6 = _iterator6.next()).done); _iteratorNormalCompletion6 = true) {
            var submodel = _step6.value;

            var config = this.constructor._submodelConfig(submodel);
            var _iteratorNormalCompletion7 = true;
            var _didIteratorError7 = false;
            var _iteratorError7 = undefined;

            try {
              for (var _iterator7 = this._registeredSubFormsFor(submodel, this.initialData)[Symbol.iterator](), _step7; !(_iteratorNormalCompletion7 = (_step7 = _iterator7.next()).done); _iteratorNormalCompletion7 = true) {
                var formId = _step7.value;

                objects[formId] = new config.object(this.rformData, formId);
              }
            } catch (err) {
              _didIteratorError7 = true;
              _iteratorError7 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion7 && _iterator7.return) {
                  _iterator7.return();
                }
              } finally {
                if (_didIteratorError7) {
                  throw _iteratorError7;
                }
              }
            }
          }
        } catch (err) {
          _didIteratorError6 = true;
          _iteratorError6 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion6 && _iterator6.return) {
              _iterator6.return();
            }
          } finally {
            if (_didIteratorError6) {
              throw _iteratorError6;
            }
          }
        }

        return objects;
      }
    }], [{
      key: '_submodelConfig',
      value: function _submodelConfig(submodel) {
        var config = this.submodelConfig && this.submodelConfig[submodel];
        if (!config) throw new Error('No configs given for submodel "' + submodel + '"');
        if (config.object) {
          config.properties = config.object.properties;
          config.type = config.object.type;
          config.submodels = config.object.submodels;
          config.submodelConfigs = config.object.collectedSubmodelConfigs;
        }
        return config;
      }
    }, {
      key: 'model',
      get: function get() {
        throw 'FormObject ' + this.name + ' needs to define a model name.';
      }
    }, {
      key: 'properties',
      get: function get() {
        throw 'FormObject ' + this.name + ' needs to define properties.';
      }
    }, {
      key: 'submodels',
      get: function get() {
        return [];
      }
    }, {
      key: 'submodelConfig',
      get: function get() {
        return {};
      }
    }, {
      key: 'ajaxAdapter',
      get: function get() {
        return _FormData2.default;
      }
    }, {
      key: 'collectedSubmodelConfigs',
      get: function get() {
        var configs = {};
        var _iteratorNormalCompletion8 = true;
        var _didIteratorError8 = false;
        var _iteratorError8 = undefined;

        try {
          for (var _iterator8 = this.submodels[Symbol.iterator](), _step8; !(_iteratorNormalCompletion8 = (_step8 = _iterator8.next()).done); _iteratorNormalCompletion8 = true) {
            var submodel = _step8.value;

            configs[submodel] = this._submodelConfig(submodel);
          }
        } catch (err) {
          _didIteratorError8 = true;
          _iteratorError8 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion8 && _iterator8.return) {
              _iterator8.return();
            }
          } finally {
            if (_didIteratorError8) {
              throw _iteratorError8;
            }
          }
        }

        return configs;
      }
    }]);

    return FormObject;
  }();

  exports.default = FormObject;


  (0, _mixin2.default)(FormObject, _Validation2.default);
});