(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'lodash/forIn', 'lodash/uniqBy', 'lodash/compact', 'lodash/cloneDeep', './predicates', '../utils/translations', '../utils/getErrors'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('lodash/forIn'), require('lodash/uniqBy'), require('lodash/compact'), require('lodash/cloneDeep'), require('./predicates'), require('../utils/translations'), require('../utils/getErrors'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.forIn, global.uniqBy, global.compact, global.cloneDeep, global.predicates, global.translations, global.getErrors);
    global.Validator = mod.exports;
  }
})(this, function (exports, _forIn, _uniqBy, _compact, _cloneDeep, _predicates, _translations, _getErrors) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _forIn2 = _interopRequireDefault(_forIn);

  var _uniqBy2 = _interopRequireDefault(_uniqBy);

  var _compact2 = _interopRequireDefault(_compact);

  var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

  var _predicates2 = _interopRequireDefault(_predicates);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
  };

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  var Validator = function () {
    function Validator(property, attrs, submodel, errorKey, defaultFilledPredicate) {
      var configurations = arguments.length <= 5 || arguments[5] === undefined ? {} : arguments[5];

      _classCallCheck(this, Validator);

      this.property = property;
      this.attrs = attrs;
      this.submodel = submodel;
      this.errorKey = errorKey;
      this.defaultFilledPredicate = defaultFilledPredicate;
      this.configurations = configurations;
    }

    // Require presence with optional additions


    _createClass(Validator, [{
      key: 'filled',
      value: function filled() {
        var _this = this;

        for (var _len = arguments.length, chosenPredicates = Array(_len), _key = 0; _key < _len; _key++) {
          chosenPredicates[_key] = arguments[_key];
        }

        if (this.defaultFilledPredicate) chosenPredicates.unshift('filled?');

        var errors = [];

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = chosenPredicates[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var predicate = _step.value;

            if ((typeof predicate === 'undefined' ? 'undefined' : _typeof(predicate)) === 'object') {
              var optionPredicates = predicate;
              (0, _forIn2.default)(predicate, function (option, predicateWithOption) {
                return _this._checkPredicate(errors, predicateWithOption, option);
              });
            } else {
              this._checkPredicate(errors, predicate);
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        return this.setErrors((0, _uniqBy2.default)((0, _compact2.default)(errors)));
      }
    }, {
      key: '_checkPredicate',
      value: function _checkPredicate(errors, predicate) {
        var option = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

        var predicateCheck = this._predicate(predicate);
        if ((typeof predicateCheck === 'undefined' ? 'undefined' : _typeof(predicateCheck)) == 'object') predicateCheck = predicateCheck.check;
        if (!predicateCheck) throw new Error('Unknown predicate "' + predicate + '"');

        if (!predicateCheck(this._validatable, option)) {
          errors.push(this._errorMessage(predicate, option));
        }
      }
    }, {
      key: '_errorMessage',
      value: function _errorMessage(predicate, option) {
        var translationOptions = this._predicate(predicate).translationOptions && this._predicate(predicate).translationOptions(option);

        var translationPath = ['errors', predicate];
        if (this._predicate(predicate).argType) {
          translationPath.push('arg');
          translationPath.push(this._predicate(predicate).argType(option));
        }

        return _translations.optionalTranslation.apply(undefined, translationPath.concat([translationOptions]));
      }
    }, {
      key: '_predicate',
      value: function _predicate(predicate) {
        return _predicates2.default[predicate] || this.configurations[predicate];
      }
    }, {
      key: 'setErrors',
      value: function setErrors(errors) {
        this.attrs[_getErrors.ERRORCONTAINER][this.errorKey] = errors;
        return errors;
      }
    }, {
      key: '_validatable',
      get: function get() {
        if (this.submodel) {
          return this.attrs[this.submodel][this.property];
        } else {
          return this.attrs[this.property];
        }
      }
    }]);

    return Validator;
  }();

  exports.default = Validator;
});