(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'lodash/merge', 'lodash/forIn', './Base'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('lodash/merge'), require('lodash/forIn'), require('./Base'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.merge, global.forIn, global.Base);
    global.JsonApi = mod.exports;
  }
})(this, function (exports, _merge, _forIn, _Base) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _merge2 = _interopRequireDefault(_merge);

  var _forIn2 = _interopRequireDefault(_forIn);

  var _Base2 = _interopRequireDefault(_Base);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  var _createClass = function () {
    function defineProperties(target, props) {
      for (var i = 0; i < props.length; i++) {
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
      }
    }

    return function (Constructor, protoProps, staticProps) {
      if (protoProps) defineProperties(Constructor.prototype, protoProps);
      if (staticProps) defineProperties(Constructor, staticProps);
      return Constructor;
    };
  }();

  function _possibleConstructorReturn(self, call) {
    if (!self) {
      throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }

    return call && (typeof call === "object" || typeof call === "function") ? call : self;
  }

  function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
      throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
    }

    subClass.prototype = Object.create(superClass && superClass.prototype, {
      constructor: {
        value: subClass,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
  }

  var JsonApiAdapter = function (_BaseAdapter) {
    _inherits(JsonApiAdapter, _BaseAdapter);

    function JsonApiAdapter() {
      _classCallCheck(this, JsonApiAdapter);

      return _possibleConstructorReturn(this, Object.getPrototypeOf(JsonApiAdapter).apply(this, arguments));
    }

    _createClass(JsonApiAdapter, [{
      key: 'handleAjaxResponse',
      value: function handleAjaxResponse(json) {
        var changes = {};

        if (json.data) {
          changes = this._changeElementFromJsonapiObject(json.data);
        }

        if (json.included) {
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = json.included[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var includedObject = _step.value;

              (0, _merge2.default)(changes, this._changeElementFromJsonapiObject(includedObject));
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }
        }

        var errors = this._deserializeErrors(json.errors);

        return [changes, errors, json.meta];
      }
    }, {
      key: '_changeElementFromJsonapiObject',
      value: function _changeElementFromJsonapiObject(object) {
        var change = {};
        change[object.type] = {};
        change[object.type][object.id] = (0, _merge2.default)(object.attributes || {}, { id: object.id });
        return change;
      }
    }, {
      key: '_collectAllJsonPropsRecursively',
      value: function _collectAllJsonPropsRecursively(attrs, config) {
        if (String(attrs).match(/^\d+$/)) {
          // given obj was an ID, not a submodel
          return { id: String(attrs), type: config.type };
        } else {
          var jsonObject = {};
          this._addModelPropsToJson(jsonObject, attrs, config);
          this._addSubmodelPropsToJson(jsonObject, attrs, config);
          return jsonObject;
        }
      }
    }, {
      key: '_buildMetaInformation',
      value: function _buildMetaInformation(attrs, config) {
        var jsonObject = {};
        var form = this.form;
        var metaFields = ['utf8', 'commit'];

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = metaFields[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var metaField = _step2.value;

            jsonObject[metaField] = form[metaField] && form[metaField].value;
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        return jsonObject;
      }
    }, {
      key: '_addModelPropsToJson',
      value: function _addModelPropsToJson(json, attrs, config) {
        json.type = config.type;
        if (config.id) json.id = String(config.id);

        if (config.properties.length) json.attributes = {};

        var _iteratorNormalCompletion3 = true;
        var _didIteratorError3 = false;
        var _iteratorError3 = undefined;

        try {
          for (var _iterator3 = config.properties[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var property = _step3.value;

            if (config.submodels.includes(property)) continue;
            json.attributes[property] = attrs[property];
          }
        } catch (err) {
          _didIteratorError3 = true;
          _iteratorError3 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
              _iterator3.return();
            }
          } finally {
            if (_didIteratorError3) {
              throw _iteratorError3;
            }
          }
        }
      }
    }, {
      key: '_addSubmodelPropsToJson',
      value: function _addSubmodelPropsToJson(json, attrs, config) {
        var _this2 = this;

        var submodelConfigs = config.submodelConfigs;


        (0, _forIn2.default)(submodelConfigs, function (innerConfig, submodel) {
          json.relationships = json.relationships || {};
          json.relationships[submodel] = {};

          if (attrs[submodel] && !attrs[submodel].length) return;
          if (innerConfig.relationship == 'oneToOne') {
            json.relationships[submodel].data = _this2._collectAllJsonPropsRecursively(attrs[submodel][0], innerConfig);
          } else {
            json.relationships[submodel].data = attrs[submodel].map(function (subset) {
              return _this2._collectAllJsonPropsRecursively(subset, innerConfig);
            });
          }
        });
      }
    }, {
      key: '_deserializeErrors',
      value: function _deserializeErrors(jsonapiErrors) {
        var errors = {};
        if (jsonapiErrors) {
          var rformData = this.rformData;
          var formId = this.formId;
          var _iteratorNormalCompletion4 = true;
          var _didIteratorError4 = false;
          var _iteratorError4 = undefined;

          try {
            for (var _iterator4 = jsonapiErrors[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
              var err = _step4.value;

              var pointer = err.source.pointer;

              var relationshipFinder = new RegExp('relationships/([^/]+)/data/(\\d+)?');

              var result = void 0;
              var path = errors;
              var errorFormId = formId;
              while (result = pointer.match(relationshipFinder)) {
                pointer = pointer.substr(result.index + result[0].length);
                var relationName = result[1];
                var optionalIndex = result[2] || 0;

                if (rformData[errorFormId]._registeredSubmodelForms && rformData[errorFormId]._registeredSubmodelForms[relationName] && rformData[errorFormId]._registeredSubmodelForms[relationName][optionalIndex]) {
                  errorFormId = rformData[errorFormId]._registeredSubmodelForms[relationName][optionalIndex];
                }

                // TODO: handle unregistered submodel - maybe use following as `else`
                // path[relationName] = path[relationName] || {}
                // path = path[relationName]
                // if (optionalIndex) {
                //   path[optionalIndex] = path[optionalIndex] || {}
                //   path = path[optionalIndex]
                // }
              }

              var attributeName = pointer.match(new RegExp('attributes/(.+)'))[1];
              path[errorFormId] = path[errorFormId] || {};
              path[errorFormId][attributeName] = path[errorFormId][attributeName] || [];
              path[errorFormId][attributeName].push(err.title);
            }
          } catch (err) {
            _didIteratorError4 = true;
            _iteratorError4 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion4 && _iterator4.return) {
                _iterator4.return();
              }
            } finally {
              if (_didIteratorError4) {
                throw _iteratorError4;
              }
            }
          }
        }
        return errors;
      }
    }, {
      key: 'requestHash',
      get: function get() {
        var options = {
          method: this.form._method.value,
          credentials: 'same-origin',
          headers: {
            'Content-Type': 'application/vnd.api+json',
            Accept: 'application/vnd.api+json'
          }
        };

        if (this.attrs) options.body = JSON.stringify(this.renderedJson);

        if (this.form.authenticity_token) options.headers['X-CSRF-TOKEN'] = this.form.authenticity_token.value;

        return options;
      }
    }, {
      key: 'renderedJson',
      get: function get() {
        var jsonObject = {
          data: this._collectAllJsonPropsRecursively(this.attrs, this.baseConfig),
          meta: this._buildMetaInformation(this.attrs, this.baseConfig)
        };
        return jsonObject;
      }
    }]);

    return JsonApiAdapter;
  }(_Base2.default);

  exports.default = JsonApiAdapter;
});