(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'lodash/isArray', 'lodash/isNil'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('lodash/isArray'), require('lodash/isNil'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.isArray, global.isNil);
    global.predicates = mod.exports;
  }
})(this, function (exports, _isArray, _isNil) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });

  var _isArray2 = _interopRequireDefault(_isArray);

  var _isNil2 = _interopRequireDefault(_isNil);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  exports.default = {
    'filled?': {
      check: function check(validatable) {
        return (0, _isArray2.default)(validatable) ? validatable.length : !(0, _isNil2.default)(validatable) && validatable !== '';
      }
    },
    'int?': {
      check: function check(validatable) {
        return (/^\d+$/.test(validatable)
        );
      }
    },
    'date?': {
      check: function check(validatable) {
        return (/^\d{4}-\d{2}-\d{2}$/.test(validatable)
        );
      }
    },
    'max_size?': {
      check: function check(validatable, option) {
        return validatable.length <= option;
      },
      translationOptions: function translationOptions(option) {
        return { num: option };
      }
    },
    'gt?': {
      check: function check(validatable, option) {
        return validatable > option;
      },
      translationOptions: function translationOptions(option) {
        return { num: option };
      }
    },
    'matches?': {
      check: function check(validatable, option) {
        return validatable == option;
      },
      translationOptions: function translationOptions(option) {
        return { num: option };
      }
    },
    'format?': {
      check: function check(validatable, regex) {
        return regex.test(validatable);
      }
    },
    'size?': {
      check: function check(validatable, range) {
        return validatable.length > range[0] && validatable.length < range[1];
      },
      argType: function argType(arg) {
        return (0, _isArray2.default)(arg) ? 'range' : 'default';
      },
      translationOptions: function translationOptions(option) {
        return {
          size_left: option[0], size_right: option[1]
        };
      }
    }
  };
});