(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports', 'lodash/assign', 'lodash/merge', 'lodash/isArray', 'lodash/isEqual', 'lodash/pickBy', 'lodash/remove', 'lodash/forIn', '../utils/getErrors', '../utils/stateNavigation'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports, require('lodash/assign'), require('lodash/merge'), require('lodash/isArray'), require('lodash/isEqual'), require('lodash/pickBy'), require('lodash/remove'), require('lodash/forIn'), require('../utils/getErrors'), require('../utils/stateNavigation'));
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports, global.assign, global.merge, global.isArray, global.isEqual, global.pickBy, global.remove, global.forIn, global.getErrors, global.stateNavigation);
    global.index = mod.exports;
  }
})(this, function (exports, _assign, _merge, _isArray, _isEqual, _pickBy, _remove, _forIn, _getErrors, _stateNavigation) {
  'use strict';

  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.initialState = undefined;
  exports.default = reducer;

  var _assign2 = _interopRequireDefault(_assign);

  var _merge2 = _interopRequireDefault(_merge);

  var _isArray2 = _interopRequireDefault(_isArray);

  var _isEqual2 = _interopRequireDefault(_isEqual);

  var _pickBy2 = _interopRequireDefault(_pickBy);

  var _remove2 = _interopRequireDefault(_remove);

  var _forIn2 = _interopRequireDefault(_forIn);

  function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
      default: obj
    };
  }

  var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
    return typeof obj;
  } : function (obj) {
    return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
  };

  var INTERNAL_KEYS = ['_savedAttributes', '_changes', 'errors'];

  var initialState = exports.initialState = {
    isSubmitting: {}
  };

  function reducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];
    var action = arguments[1];
    var type = action.type;

    var newState = (0, _assign2.default)({}, state);

    var _ret = function () {
      switch (type) {
        case '_RFORM_SET_FORM':
          newState[action.formId] = (0, _merge2.default)(action.formData, { _savedAttributes: action.formData, _changes: [] });
          return {
            v: newState
          };

        case '_RFORM_SET_FORM_SAVED':
          newState[action.formId] = (0, _merge2.default)(state[action.formId], {
            _savedAttributes: (0, _pickBy2.default)(state[action.formId], function (value, key) {
              return !INTERNAL_KEYS.includes(key);
            })
          });
          newState[action.formId]._changes = [];
          return {
            v: newState
          };

        case '_RFORM_UPDATE_FORM_ATTRIBUTE':
          var attribute = action.attribute;
          var formId = action.formId;
          var submodelPath = action.submodelPath;

          var changed = action.changed;
          var formBasePath = newState[formId];
          var submodelBasePath = (0, _stateNavigation.navigateThroughSubmodels)(formBasePath, submodelPath, true);
          submodelBasePath[attribute] = action.value;

          changed = analyzeChanges(changed, submodelBasePath, attribute);

          var changesBasePath = (0, _stateNavigation.navigateThroughSubmodels)(formBasePath._changes, submodelPath, true, []);
          if (changed === true && !changesBasePath.includes(attribute)) changesBasePath.push(attribute);
          if (changed === false && changesBasePath.includes(attribute)) (0, _remove2.default)(changesBasePath, function (value) {
            return value == attribute;
          });

          return {
            v: newState
          };

        case '_RFORM_UPDATE_ERROR':
          if (action.onlyKey) {
            newState[action.formId][_getErrors.ERRORCONTAINER] = newState[action.formId][_getErrors.ERRORCONTAINER] || {};
            newState[action.formId][_getErrors.ERRORCONTAINER][action.onlyKey] = action.errors[action.onlyKey];
          } else {
            newState[action.formId][_getErrors.ERRORCONTAINER] = action.errors;
          }
          return {
            v: newState
          };

        case '_RFORM_SUBMIT_AJAX_FORM_REQUEST':
          newState.isSubmitting[action.formId] = true;
          return {
            v: newState
          };

        case '_RFORM_SUBMIT_AJAX_FORM_RETURN':
          newState.isSubmitting[action.formId] = false;
          return {
            v: newState
          };

        case '_RFORM_SUBMIT_AJAX_FORM_BUTTON_REQUEST':
          newState.isSubmitting[action.method + ':' + action.action] = true;
          return {
            v: newState
          };

        case '_RFORM_SUBMIT_AJAX_FORM_BUTTON_RETURN':
          newState.isSubmitting[action.method + ':' + action.action] = false;
          return {
            v: newState
          };

        case '_RFORM_HANDLE_AJAX_RESPONSE':
          var responseChanges = {};
          if (action.formErrorHash) {
            (0, _forIn2.default)(action.formErrorHash, function (errors, formId) {
              responseChanges[formId] = state[formId];
              responseChanges[formId][_getErrors.ERRORCONTAINER] = errors;
            });
          }

          newState = (0, _merge2.default)(newState, responseChanges);
          return {
            v: newState
          };

        case '_RFORM_REGISTER_SUBMODEL_FORM':
          var registeringFormId = action.registeringFormId;
          var submodelName = action.submodelName;
          var registeredSubmodelFormId = action.registeredSubmodelFormId;

          newState[registeringFormId]._registeredSubmodelForms = newState[registeringFormId]._registeredSubmodelForms || {};
          newState[registeringFormId]._registeredSubmodelForms[submodelName] = newState[registeringFormId]._registeredSubmodelForms[submodelName] || [];

          newState[registeringFormId]._registeredSubmodelForms[submodelName].push(registeredSubmodelFormId);
          return {
            v: newState
          };

        case '_RFORM_UNREGISTER_SUBMODEL_FORM':
          (0, _remove2.default)(newState[action.unregisteringFormId]._registeredSubmodelForms[action.submodelName], function (element) {
            return element == action.unregisteredSubmodelFormId;
          });
          return {
            v: newState
          };

        default:
          return {
            v: newState
          };
      }
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
  }

  function analyzeChanges(changed, submodelBasePath, attribute) {
    if (changed === undefined) {
      if (submodelBasePath._savedAttributes[attribute] === undefined) changed = !!submodelBasePath[attribute];else {
        var newValue = submodelBasePath[attribute];
        // if (isArray(newValue)) newValue = newValue.sort()
        var currentValue = submodelBasePath._savedAttributes[attribute];
        // if (isArray(currentValue)) currentValue = currentValue.sort()
        changed = !(0, _isEqual2.default)(newValue, currentValue);
      }
    }
    return changed;
  }
});